/*
 * @Author: zong.wang01@hand-china.com
 * @Date: 2024-07-30 09:41:54
 * @LastEditors: zong.wang01@hand-china.com
 * @LastEditTime: 2024-08-09 11:10:49
 * @Version: 1.0.0
 * @Description: 
 * @Copyright: Copyright (c) 2021, Hand-RongJing
 */
import Vue from 'vue';
import {Button} from 'vant'
import layouts from '../layout';
import {ColumnBuilder, TabButtonBuilder, ComponentBuilder} from './utils.jsx'
import QueryForm from './QueryForm';
import DTable from './DTable';
import DForm from './DForm';
 
const ConfigRenderComponent = {
  // functional: true,
  // Props 是可选的
  components: {
    QueryForm,
    DTable,
    DForm,
  },
  props: {
    needBack: { // 是否需要返回键
      type: Boolean,
      default: true
    },
    staticLayoutTabs: {
      type: Array,
      default: () => [],
    }
  },
  data () {
    return {
      configInfo: {}, // 配置信息
      layoutComponents: ['HBox', 'VBox', 'Tabs', 'TabPane', 'NavigationBar'],
      queryParams: {}, // 路由参数
    }
  },
  inject: ['config'],
  created() {
    this.queryParams = {
      ...this.$route.params,
      ...this.$route.query
    }
  },
  mounted() {
    this.configInfo = this.config();
  },
  methods: {
    getTabs(layoutTabs, state, layoutCode, createElement) {
      return layoutTabs.map(layoutTab => {
        const parentTabCode = layoutTab.tabCode;
        // 布局类型
        const layoutName = layoutTab.tabType;
        // 静态嵌入布局
        if (layoutName === 'Layout' && layoutTab.tabStatusType === 'STATIC') {
          const currentStaticLayoutTabs = this.staticLayoutTabs.filter(
            value => value.tabCode === parentTabCode
          );
          if (currentStaticLayoutTabs.length > 0 && currentStaticLayoutTabs[0].component) {
            const StaticComponent = currentStaticLayoutTabs[0].component;
            return [<StaticComponent {...this} />];
          }
          return [];
        }
        if (!layoutTab.childrenTabs || layoutTab.childrenTabs.length === 0) {
          return [];
        }
        // 组件描述
        const componentNames = [];
        let components;
        if (this.layoutComponents.some(item => layoutTab.childrenTabs[0].tabType === item)) {
          layoutTab.childrenTabs.forEach(childrenTab => {
            // componentNames.push(childrenTab.description);
            componentNames.push({code: childrenTab.tabCode, label: childrenTab.description})
          });
          components = this.getTabs(layoutTab.childrenTabs, state, layoutCode, createElement);
        } else {
          components = layoutTab.childrenTabs.map(childrenTab => {
            const {
              tabType,
              tabCode,
              baseTable,
              parentTable,
              tabFields,
              tabButtons,
              description
            } = childrenTab;
            // componentNames.push(description);
            componentNames.push({code: tabCode, label: description});
            
            const columns = new ColumnBuilder()
              .withFields(tabFields)
              .withType(tabType)
              .withTabCode(tabCode)
              // .withTableFooterRenderers(tableFooterRenderers)
              .withLayoutCode(childrenTab.layoutCode || layoutCode)
              .withParentTabCode(parentTabCode)
              // .withComponentRenderers(componentRenderers)
              .build();
             
            const findField = find(tabFields, { formatCascadeFlag: 'Y' });
            
            return new ComponentBuilder()
              // .withDataSet(dataSet)
              // .withHistoryDataSet(dataSets[`history_${dataSetName}`])
              // .withDifference(differenceMap[dataSetName])
              .withCascadeKey(findField && findField.columnName)
              // .withHistoryButtonName(historyButtonName)
              // .withCurrentButtonName(currentButtonName)
              .withRouteParams(this.queryParams)
              .withCreateElement(createElement)
              .withTabType(tabType)
              .withColumns(columns)
              // .withButtons(buttons)
              .withLayoutCode(childrenTab.layoutCode || layoutCode)
              .withTabCode(childrenTab.tabCode)
              .withProps(childrenTab)
              // .withComponentHeaderExtras(componentHeaderExtras)
              .build();
          });
        }
        const Layout = this.getLayout(layoutName);
        return <Layout componentNames={componentNames} components={components}/>
        // return new LayoutBuilder()
        //   .withLayout(this.getLayout(layoutName))
        //   .withTabCode(layoutTab.tabCode)
        //   // .withTabMap(tabMap)
        //   .withComponents(components)
        //   .withComponentNames(componentNames)
        //   // .withComponentFunctions(componentFunctions)
        //   .build();
      });
    },
    getLayout(layoutName) {
      return layouts[layoutName];
    }
  },
  // 为了弥补缺少的实例
  // 提供第二个参数作为上下文
  render: function (h, context) {
    console.log('context', context, this.configInfo)
    const { layoutCode, layoutTabs = [] } = this.configInfo;
    return <div style="height: 100%">{layoutTabs.length ? this.getTabs(layoutTabs, this, layoutCode, h) : null}</div>;
  }
};
 
export default ConfigRenderComponent;